#include "common.h"

struct av 
{
	float4 	pos	: POSITION;	// (float,float,float,1)
	float4 	nc	: NORMAL;	// (float,float,float,clr)
	float4 	misc	: TEXCOORD0;	// (u(Q),v(Q),frac,???)
};

struct vf
{
	float4 HPOS	: POSITION;
	float3 COL0	: COLOR0;
	float2 TEX0	: TEXCOORD0;
	float  fog	: FOG;
};

uniform float4 		consts;		// {1/quant,1/quant,???,???}
uniform float4 		wave; 		// cx,cy,cz,tm
uniform float4 		wind; 		// direction2D
uniform float4		c_bias;		// + color
uniform float4		c_scale;	// * color
uniform float2 		c_sun;		// x=*, y=+

vf main (av v)
{
	vf 		o;

	// Transform to world coords
	float3 	pos	= mul	(m_W, v.pos);

	// 
	float 	base 	= m_W._24;			// take base height from matrix
	float 	dp	= sin   (wave.w+dot(pos,(float3)wave));
	float 	H 	= pos.y - base;			// height of vertex (scaled, rotated, etc.)
	float 	frac 	= v.misc.z*consts.x;		// fractional (or rigidity)
	float 	inten 	= H * dp;			// intensity
	float2 	result	= calc_xz_wave	(wind.xz*inten, frac);
	float4 	f_pos 	= float4(pos.x+result.x, pos.y, pos.z+result.y, 1);

	// Calc fog
	o.fog 		= calc_fogging 	(f_pos);

	// Final xform
	o.HPOS		= mul		(m_VP, f_pos);

	// Lighting
	float3 	N 	= normalize 	(mul (m_W,  unpack_normal(v.nc)));
	float 	L_base 	= v.nc.w;								// base hemisphere
	float4 	L_unpack= c_scale*L_base+c_bias;						// unpacked and decompressed
	float3 	L_rgb 	= L_lmap_color*L_unpack.xyz;								// precalculated RGB lighting
	float3 	L_hemi 	= L_hemi_color*L_unpack.w;						// hemisphere
	float3 	L_sun 	= L_sun_color*(L_base*c_sun.x+c_sun.y)*max(0,dot(N,-L_sun_dir_w));	// sun
	float3 	L_final	= L_rgb + L_hemi + L_sun + L_ambient;
	o.COL0		= L_final;

	// final xform, color, tc
	o.TEX0.xy	= (v.misc * consts).xy;

	return o;
}
